%constrained.m use a policy-function iteration procedure to approximate the equilibrium of  the unregulated  small open economy with a flow 
%collateral constraint driven by interest-rate and tradable-endowment shocks as presented in 
%``Is Optimal Capital-Control Policy Countercyclical In Open-Economy  Models With Collateral Constraints?'' by Stephanie Schmitt-Groh and Martn Uribe, July 2016. 
%The algorithm accounts for  the possibility that for
%some current states there is no choice of next-period debt 
%for which consumption of tradables is positive and the collateral constraint is satisfied. This is done by calling pathfinder.m
%The present model is cast at annual frequency.
%  Stephanie Schmitt-Groh and Martn Uribe, July  2016.

clear all,  format compact

eqm_selection_criterion = 'c';
%the other equilibrium criteria are 'b' and 'a'. All should deliver the same policy functions if the equilibrium is unique. This is indeed satisfied  under the present calibration, so no need to run  the program under criteria 'b' or 'a'. 
% For more on the possibility of multiple equilibria in open-economy models with flow collateral constraints see
%``Multiple Equilibria in Open Economy Models with Collateral Constraints: Overborrowing Revisited,'' by Stephanie Schmitt-Groh and Martn Uribe, May 2016. 

load annual_tpm.mat pai ygrid rgrid; %produced by running 
%annual_tpm.m
%in
%z:\joint\macroprudential\rshocks

ny = numel(ygrid); %number of grid points for the joint process of tradable output and  the country interest rate (yT,r)

nd = 800;% # of grid points for debt, d

filename = ['constrained_' eqm_selection_criterion] %name of .mat file where the output of this program is saved

%Calibration
rstar = (1+0.0316)^4-1;  %average Argentine quarterly interest rate. Estimated using data from 1983:1 to 2001:4 in the program  c:\data\uribe\default\data\exogenous_process.m (we cut the post 2001 period because of the great default of 2001, since our current model does not feature default).

%Calibrate the subjective discount factor to ensure that the relative impatience factor is the same as in Bianchi (AER, 2011)
impatience_in_bianchi = 0.91*1.04;
betta = impatience_in_bianchi/(1+rstar); 

a =  0.31;%weight on traded consumption in the CES aggregator 

yN = 1; %nontraded output

xi = 0.83;% Elasticity of substitution between traded and nontraded goods

sigg = 2; %intertemporal elasticity of consumption

kapa = 1.04*0.32; %Parameter defining the collateral constraint (debt is bounded above by kapa% of annual output)

yTgridlevel = exp(ygrid(:)); %grid of level of tradable output
rgridlevel = exp(rgrid)*(1+rstar)-1; %grid of level of country interest rate

NdL = min(yTgridlevel)*(1+max(rgridlevel))/max(rgridlevel); %natural debt limit

%Lower and upper bounds for debt grid
dlower = -0.5;%-2.5;%-3;% 
dupper =  1.5;%2;%

%debt grid
dgrid = linspace(dlower,dupper,nd)';
dd = dgrid(2) - dgrid(1);

n = ny*nd; %total number of states

%Let yT, r, and d  be 3 ny-by-nd matrices containing values  of the current state
yT = repmat(yTgridlevel,1,nd);  
r = repmat(rgridlevel,1,nd);  
d = repmat(dgrid',ny,1);

dix = repmat(1:nd,ny,1); %ny-by-nd matrix of indices of the current debt state

%n-by-nd matrix of tradable consumption as a function of the current state (n in total) and  possible choices of next-period debt (nd in total)
bsxfun(@rdivide,dgrid',1+r(:));
cTtry = bsxfun(@plus,ans,yT(:)-d(:));

if min(max(cTtry,[],2))<=0
error('Natural debt limit violated')
end

%n-by-nd matrix of relative price of nontradables as a function of the current state (n in total) and possible values for next-period debt (nd in total)
ptry  = (1-a)/a*(cTtry/yN).^(1/xi);

%n-by-nd matrix of the value of collateral  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
collateraltry = kapa*bsxfun(@plus,yT(:),yN*ptry);

%n-by-nd matrix of the slackness of the collateral constraint  as a function of the current state (n in total) and possible values for next-period debt (nd in total). 
slacktry = bsxfun(@minus,collateraltry,dgrid');

%Gridize  values of  slacktry matrix for   which the collateral constraint is binding  by setting them equal to 0
aux1 = sign(slacktry);
aux2 = sign([slacktry(:,1) slacktry(:,1:end-1)]);
aux3 = sign([ slacktry(:,2:end) slacktry(:,end)]);
slacktry1=slacktry; 
slacktry(aux1.*aux2==-1 & slacktry1<0) = 0;
slacktry(aux1.*aux3==-1 & slacktry1<0) = 0;

%Identify the indices  at which slack is nil (binding collateral constraint). 
%If for the current state there exists exactly 1 value of next-period debt for which slack is 0,  
%collect this index in bind1ix. If for the current state there exist  two binding points, the larger is collected in bind2ix, 
%and the other in bind1ix.  
%The vectors (bind1i,bind1j) are the subindex representation of bind1ix, and similarly for (bind2i,bind2j). 
%Note: the reason why there can be exactly 1 binding point for a given state (as opposed to either 0 or 2) 
%is that we have already set to NaN all entries of slack associated with cT<=0 (which could have contained binding points); 
%thus  bind2i is a subset of bind1i. 
[aux1,aux2]= min(abs(slacktry),[],2);
bind1i = find(aux1==0);
bind1j = aux2(bind1i);
bind1ix = sub2ind([n nd],bind1i,bind1j);
find(slacktry==0);
bind2ix = setdiff(ans,bind1ix);
[bind2i,bind2j] = ind2sub([n nd],bind2ix);

%n-by-nd matrix of consumption of the composite good  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
ctry = (a * cTtry.^(1-1/xi) + (1-a) * yN.^(1-1/xi)).^(1/(1-1/xi));

%n-by-nd matrix of marginal utility of tradable consumption  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
latry = a*ctry.^(-sigg) .* (cTtry./ctry).^(-1/xi);
latry(cTtry<=0) = 1e6;

%n-by-nd matrix of lambda_t/(1+r_t)  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
LHSEulertry = bsxfun(@rdivide,latry,1+r(:));

%initializations
la = zeros(ny,nd); %marginal utilliy of tradable consumption
dpix  = zeros(ny,nd); %debt policy function in index form (entries take values between 1 and nd)
dpix(:) = dix ; %set initial guess for debt policy funciton (index form) to current  state 
dpixnew  = zeros(ny,nd); %iupdate  guess 
distTol = 1e-8; %convergence criterion
dist_dpix = inf;  %distance criterion

Ztry = pathfinder(dupper, dlower, nd, rstar, betta, a, yN,xi, sigg, kapa);%This step rules out picking values for d_{t+1} that put the economy with positive probability into a state tomorrow in which either consumption is negative or the collateral constraint is violated. 
while dist_dpix>0 

picker = sub2ind([n nd],(1:n)',dpix(:));
la(:) = latry(picker); %picks the equilibrium  choice out of any n-by-nd matrix  containing for each state (n in total) possible policy choices (nd in total)

%n-by-nd matrix of mu_t*la_t as a function of the current state (n in total) and possible values for next-period debt (nd in total)
MUtry = LHSEulertry - betta*repmat(pai *  la,nd,1); 

%Find MUtry =0, by identifying sign change along rows
aux1 = sign(MUtry);
aux2 = sign([MUtry(:,1) MUtry(:,1:end-1)]);
MUtry(aux1~=aux2 & ~isnan(aux1.*aux2)) = 0;

AMUtry = abs(MUtry);
AMUtry(slacktry<0 ) = NaN;
AMUtry = AMUtry.*Ztry;
[~, dpixnew(:)] = min(AMUtry,[],2);

%equilibrium selection  criterion (a)
if strcmp(eqm_selection_criterion,'a')
picker_a = sub2ind([n nd],(1:n)',dpixnew(:));
mu_neg_ix=find(MUtry(picker_a)<0); 
[neg_ix, ixbind]=intersect(bind1i, mu_neg_ix);
dpixnew(neg_ix) =bind1j(ixbind); 
end

%equilibrium selection  criterion (c)
if strcmp(eqm_selection_criterion,'c')
find(MUtry(bind1ix)>=0);
dpixnew(bind1i(ans)) = bind1j(ans);
end

%equilibrium selection  criterion (b)
if strcmp(eqm_selection_criterion,'b')
find(MUtry(bind1ix)>=0);
dpixnew(bind1i(ans)) = bind1j(ans);
find(MUtry(bind2ix)>=0);
dpixnew(bind2i(ans)) = bind2j(ans);
end

%Note. To verify that  for all states that are visited in equilibrium all equilibrium 
%conditions are satisfied, it must be verified that after running the program simu.m,
%(1) the vector MU, containing the equilibrium realizations of the multiplier mu_t on the collateral constraint contains only nonnegative elements. 
%(2) It must also be verified that the vector MU.*SLACK, where SLACK contains the equilibrium realizations of the slackness of the collateral constraint, contains only zero values. 
%(3) that  CT is strictly positive,  and 
%(4) that SLACK is non-negative.   
%The script simu.m  confirms the satisfaction of these conditions with a tolerance of 1e6 (i.e., by setting T=1e6) elements for MU,  SLACK, and CT. 

dist_dpix = max(abs(dpixnew(:)-dpix(:)));
xR = find(dpix~=dpixnew); %number of states that have not yet converged (for monitoring only, not used in approximation)

%Apply a one-step-at-a-time  updating criterion for the policy funciton
dpix = dpix + sign(dpixnew-dpix);

disp([dist_dpix  numel(xR)])

end %while dist_dpix>0

p = zeros(ny,nd);
p(:) = ptry(picker); %equilibrium price of nontradables 

dp = dgrid(dpix); %policy rule for debt

%compute the value funciton
v = zeros(ny,nd);
vnew = v;
u = (ctry(picker).^(1-sigg)-1)/(1-sigg);
distv = 1;
while distv>distTol
pai*v;
repmat(ans,nd,1);
vnew(:) = u+betta*ans(picker);
distv = max(abs(v(:)-vnew(:)));
v = vnew;
end %while distv>distTol

c = zeros(ny,nd);
c(:) = ctry(picker); %equilibrium composite consumption

cT = zeros(ny,nd);
cT(:) = cTtry(picker); %equilibrium tradable consumption 

%equilibrium slackness
slack = zeros(ny,nd);
slack(:) = slacktry(picker);

%equilibrium mu
mu = zeros(ny,nd);
mu(:) = MUtry(picker);

%crisis in the present period
slacktry==0;
crisis = zeros(ny,nd);
crisis(:) = ans(picker); 

eval(['save ' filename '.mat filename nd ny  a r betta sigg   pai   dgrid ygrid rgrid yTgridlevel rgridlevel   la dpix dp v crisis yT yN cT c kapa  picker  dupper dlower dd rstar r distTol dist_dpix d p  slack mu  xi a kapa bind1i bind2i bind1ix bind2ix  bind1j bind2j  '])