% ramsey.m uses a modified value function  iteration procedure to approximate the Ramsey equilibrium 
% of a two-sector open economy in which borrowing is limited by a fraction of output as presented in the paper 
% ``Is Optimal Capital-Control Policy Countercyclical In Open-Economy  Models With Collateral Constraints?'' by Stephanie Schmitt-Groh and Martn Uribe, July 2016. 
% The Ramsey planner is assumed to have access to capital controls. 
% Aggregate fluctuations are driven by random variations in the country  interest-rate  and tradable endowment. 
% The calibration is at a annual  frequency. 
%  Stephanie Schmitt-Groh and Martn Uribe,  July 2016.

clear all,  format compact

load annual_tpm.mat pai ygrid rgrid; %produced by running 
%annual_tpm.m
%in
%z:\joint\macroprudential\rshocks

ny = numel(ygrid); %number of grid points for the joint process of tradable output and  the country interest rate (yT,r)

nd = 800;% # of grid points for debt, d

filename = 'ramsey';



%Calibration
rstar = (1+0.0316)^4-1;  %average Argentine quarterly interest rate. Estimated using data from 1983:1 to 2001:4 in the program  c:\data\uribe\default\data\exogenous_process.m (we cut the post 2001 period because of the great default of 2001, since our current model does not feature default).

impatience_in_bianchi = 0.91*1.04;

betta = impatience_in_bianchi/(1+rstar); %0.9635^4 discount factor

a =  0.31;%weight on traded consumption in the CES aggregator 

yN = 1; %nontraded output

xi = 0.83;% Elasticity of substitution between traded and nontraded goods

sigg = 2; %intertemporal elasticity of consumption

kapa = 1.04*0.32; %Parameter defining the collateral constraint (debt is bounded above by 30% of annual output)

yTgridlevel = exp(ygrid(:)); %grid of level of tradable output
rgridlevel = exp(rgrid)*(1+rstar)-1; %grid of level of country interest rate

NdL = min(yTgridlevel)*(1+max(rgridlevel))/max(rgridlevel); %natural debt limit

%Lower and upper bounds for debt grid
dlower = -0.5;%-2.5;%-3;% 
dupper = 1.5;%

%debt grid
dgrid = linspace(dlower,dupper,nd)';
dd = dgrid(2) - dgrid(1);

n = ny*nd; %total number of states

%Let yT, r, and d  be 3 ny-by-nd matrices containing values  of the current state
yT = repmat(yTgridlevel,1,nd);  
r = repmat(rgridlevel,1,nd);  
d = repmat(dgrid',ny,1);

dix = repmat(1:nd,ny,1); %ny-by-nd matrix of indices of the current debt state

%n-by-nd matrix of tradable consumption as a function of the current state (n in total) and  possible choices of next-period debt (nd in total)
bsxfun(@rdivide,dgrid',1+r(:));
cTtry = bsxfun(@plus,ans,yT(:)-d(:));

if min(max(cTtry,[],2))<=0
error('Natural debt limit violated')
end

%n-by-nd matrix of relative price of nontradables as a function of the current state (n in total) and possible values for next-period debt (nd in total)
ptry  = (1-a)/a*(cTtry/yN).^(1/xi);

%n-by-nd matrix of the value of collateral  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
collateraltry = kapa*bsxfun(@plus,yT(:),yN*ptry);

%n-by-nd matrix of the slackness of the collateral constraint  as a function of the current state (n in total) and possible values for next-period debt (nd in total). 
slacktry = bsxfun(@minus,collateraltry,dgrid');

%Gridize  values of  slacktry matrix for   which the collateral constraint is binding  by setting them equal to 0
aux1 = sign(slacktry);
aux2 = sign([slacktry(:,1) slacktry(:,1:end-1)]);
aux3 = sign([ slacktry(:,2:end) slacktry(:,end)]);
slacktry1=slacktry; 
slacktry(aux1.*aux2==-1 & slacktry1<0) = 0;
slacktry(aux1.*aux3==-1 & slacktry1<0) = 0;

%n-by-nd matrix of consumption of the composite good  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
ctry = (a * cTtry.^(1-1/xi) + (1-a) * yN.^(1-1/xi)).^(1/(1-1/xi));

%n-by-nd matrix of marginal utility of tradable consumption  as a function of the current state (n in total) and possible values for next-period debt (nd in total)
utry = (ctry.^(1-sigg)-1)/(1-sigg);

utry(cTtry<=0 | slacktry<0) = -1e14;

%initializations
v = zeros(ny,nd); %marginal utilliy of tradable consumption
vnew = zeros(ny,nd);
dpix  = zeros(ny,nd); %debt policy function in index form (entries take values between 1 and nd)
dpix(:) = dix; %set initial guess for debt policy funciton (index form) to current  state 
dpixnew  = zeros(ny,nd); %iupdate  guess 
distTol = 1e-8; %convergence criterion
dist = inf;  %distance criterion

while dist>distTol 

pai*v;
repmat(ans,nd,1);
utry + betta*ans;
[vnew(:), dpixnew(:)] = max(ans,[],2);

dist_dpix = max(abs(dpixnew(:)-dpix(:)));
dist_v = max(abs(v(:)-vnew(:)));
dist = dist_dpix + dist_v;

xR = numel(find(dpix~=dpixnew)); %number of states that have not yet converged

v = vnew;
dpix = dpixnew;

disp([dist_dpix  xR]);
disp(dist_v);

end %while dist>distTol

picker = sub2ind([n nd],(1:n)',dpix(:));

p = zeros(ny,nd);
p(:) = ptry(picker); %equilibrium price of nontradables 
dp = dgrid(dpix); %equilibrium next-period debt

c = zeros(ny,nd);
c(:) = ctry(picker); %equilibrium composite consumption

cT = zeros(ny,nd);
cT(:) = cTtry(picker); %equilibrium tradable consumption 

la = a*c.^(-sigg) .* (cT./c).^(-1/xi);

%equilibrium slackness
slack = zeros(ny,nd);
slack(:) = slacktry(picker);

%crisis in the present period
slacktry==0;
crisis = zeros(ny,nd);
crisis(:) = ans(picker); 

%Capital control tax rate
Elap = zeros(ny,nd);
pai*la;
repmat(ans,nd,1);
Elap(:) = ans(picker);
tau = 1- betta*(1+r).*Elap./la;

eval(['save ' filename '.mat filename nd ny   r betta sigg   pai   dgrid ygrid rgrid yTgridlevel rgridlevel   la dpix dp v crisis yT yN cT c kapa  picker  dupper dlower dd rstar r distTol dist_dpix d p tau slack'])